@@ -3,7 +3,7 @@
  *  Implementation of GiNaC's sums of expressions. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,15 +20,19 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-#include <stdexcept>
-
 #include "add.h"
 #include "mul.h"
 #include "archive.h"
 #include "operators.h"
 #include "matrix.h"
 #include "utils.h"
+#include "clifford.h"
+#include "ncmul.h"
+
+#include <iostream>
+#include <limits>
+#include <stdexcept>
+#include <string>
 
 namespace GiNaC {
 
@@ -45,7 +49,6 @@ GINAC_IMPLEMENT_REGISTERED_CLASS_OPT(add, expairseq,
 
 add::add()
 {
-	tinfo_key = TINFO_add;
 }
 
 //////////
@@ -56,7 +59,6 @@ add::add()
 
 add::add(const ex & lh, const ex & rh)
 {
-	tinfo_key = TINFO_add;
 	overall_coeff = _ex0;
 	construct_from_2_ex(lh,rh);
 	GINAC_ASSERT(is_canonical());
@@ -64,7 +66,6 @@ add::add(const ex & lh, const ex & rh)
 
 add::add(const exvector & v)
 {
-	tinfo_key = TINFO_add;
 	overall_coeff = _ex0;
 	construct_from_exvector(v);
 	GINAC_ASSERT(is_canonical());
@@ -72,7 +73,6 @@ add::add(const exvector & v)
 
 add::add(const epvector & v)
 {
-	tinfo_key = TINFO_add;
 	overall_coeff = _ex0;
 	construct_from_epvector(v);
 	GINAC_ASSERT(is_canonical());
@@ -80,7 +80,6 @@ add::add(const epvector & v)
 
 add::add(const epvector & v, const ex & oc)
 {
-	tinfo_key = TINFO_add;
 	overall_coeff = oc;
 	construct_from_epvector(v);
 	GINAC_ASSERT(is_canonical());
@@ -88,7 +87,6 @@ add::add(const epvector & v, const ex & oc)
 
 add::add(std::auto_ptr<epvector> vp, const ex & oc)
 {
-	tinfo_key = TINFO_add;
 	GINAC_ASSERT(vp.get()!=0);
 	overall_coeff = oc;
 	construct_from_epvector(*vp);
@@ -99,7 +97,7 @@ add::add(std::auto_ptr<epvector> vp, const ex & oc)
 // archiving
 //////////
 
-DEFAULT_ARCHIVING(add)
+GINAC_BIND_UNARCHIVER(add);
 
 //////////
 // functions overriding virtual functions from base classes
@@ -169,37 +167,32 @@ void add::do_print_csrc(const print_csrc & c, unsigned level) const
 	if (precedence() <= level)
 		c.s << "(";
 	
-	// Print arguments, separated by "+"
+	// Print arguments, separated by "+" or "-"
 	epvector::const_iterator it = seq.begin(), itend = seq.end();
+	char separator = ' ';
 	while (it != itend) {
 		
-		// If the coefficient is -1, it is replaced by a single minus sign
-		if (it->coeff.is_equal(_ex1)) {
+		// If the coefficient is negative, separator is "-"
+		if (it->coeff.is_equal(_ex_1) || 
+			ex_to<numeric>(it->coeff).numer().is_equal(*_num_1_p))
+			separator = '-';
+		c.s << separator;
+		if (it->coeff.is_equal(_ex1) || it->coeff.is_equal(_ex_1)) {
 			it->rest.print(c, precedence());
-		} else if (it->coeff.is_equal(_ex_1)) {
-			c.s << "-";
+		} else if (ex_to<numeric>(it->coeff).numer().is_equal(*_num1_p) ||
+				 ex_to<numeric>(it->coeff).numer().is_equal(*_num_1_p))
+		{
 			it->rest.print(c, precedence());
-		} else if (ex_to<numeric>(it->coeff).numer().is_equal(*_num1_p)) {
-			it->rest.print(c, precedence());
-			c.s << "/";
-			ex_to<numeric>(it->coeff).denom().print(c, precedence());
-		} else if (ex_to<numeric>(it->coeff).numer().is_equal(*_num_1_p)) {
-			c.s << "-";
-			it->rest.print(c, precedence());
-			c.s << "/";
+			c.s << '/';
 			ex_to<numeric>(it->coeff).denom().print(c, precedence());
 		} else {
 			it->coeff.print(c, precedence());
-			c.s << "*";
+			c.s << '*';
 			it->rest.print(c, precedence());
 		}
 		
-		// Separator is "+", except if the following expression would have a leading minus sign or the sign is sitting in parenthesis (as in a ctor)
 		++it;
-		if (it != itend
-		 && (is_a<print_csrc_cl_N>(c) || !it->coeff.info(info_flags::real)  // sign inside ctor arguments
-		  || !(it->coeff.info(info_flags::negative) || (it->coeff.is_equal(*_num1_p) && is_exactly_a<numeric>(it->rest) && it->rest.info(info_flags::negative)))))
-			c.s << "+";
+		separator = '+';
 	}
 	
 	if (!overall_coeff.is_zero()) {
@@ -256,7 +249,7 @@ bool add::info(unsigned inf) const
 
 int add::degree(const ex & s) const
 {
-	int deg = INT_MIN;
+	int deg = std::numeric_limits<int>::min();
 	if (!overall_coeff.is_zero())
 		deg = 0;
 	
@@ -273,7 +266,7 @@ int add::degree(const ex & s) const
 
 int add::ldegree(const ex & s) const
 {
-	int deg = INT_MAX;
+	int deg = std::numeric_limits<int>::max();
 	if (!overall_coeff.is_zero())
 		deg = 0;
 	
@@ -291,17 +284,31 @@ int add::ldegree(const ex & s) const
 ex add::coeff(const ex & s, int n) const
 {
 	std::auto_ptr<epvector> coeffseq(new epvector);
+	std::auto_ptr<epvector> coeffseq_cliff(new epvector);
+	char rl = clifford_max_label(s);
+	bool do_clifford = (rl != -1);
+	bool nonscalar = false;
 
 	// Calculate sum of coefficients in each term
 	epvector::const_iterator i = seq.begin(), end = seq.end();
 	while (i != end) {
 		ex restcoeff = i->rest.coeff(s, n);
-		if (!restcoeff.is_zero())
+ 		if (!restcoeff.is_zero()) {
+ 			if (do_clifford) {
+ 				if (clifford_max_label(restcoeff) == -1) {
+ 					coeffseq_cliff->push_back(combine_ex_with_coeff_to_pair(ncmul(restcoeff, dirac_ONE(rl)), i->coeff));
+				} else {
+ 					coeffseq_cliff->push_back(combine_ex_with_coeff_to_pair(restcoeff, i->coeff));
+					nonscalar = true;
+ 				}
+			}
 			coeffseq->push_back(combine_ex_with_coeff_to_pair(restcoeff, i->coeff));
+		}
 		++i;
 	}
 
-	return (new add(coeffseq, n==0 ? overall_coeff : _ex0))->setflag(status_flags::dynallocated);
+	return (new add(nonscalar ? coeffseq_cliff : coeffseq,
+	                n==0 ? overall_coeff : _ex0))->setflag(status_flags::dynallocated);
 }
 
 /** Perform automatic term rewriting rules in this class.  In the following
@@ -408,6 +415,42 @@ ex add::conjugate() const
 	return *this;
 }
 
+ex add::real_part() const
+{
+	epvector v;
+	v.reserve(seq.size());
+	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i)
+		if ((i->coeff).info(info_flags::real)) {
+			ex rp = (i->rest).real_part();
+			if (!rp.is_zero())
+				v.push_back(expair(rp, i->coeff));
+		} else {
+			ex rp=recombine_pair_to_ex(*i).real_part();
+			if (!rp.is_zero())
+				v.push_back(split_ex_to_pair(rp));
+		}
+	return (new add(v, overall_coeff.real_part()))
+		-> setflag(status_flags::dynallocated);
+}
+
+ex add::imag_part() const
+{
+	epvector v;
+	v.reserve(seq.size());
+	for (epvector::const_iterator i=seq.begin(); i!=seq.end(); ++i)
+		if ((i->coeff).info(info_flags::real)) {
+			ex ip = (i->rest).imag_part();
+			if (!ip.is_zero())
+				v.push_back(expair(ip, i->coeff));
+		} else {
+			ex ip=recombine_pair_to_ex(*i).imag_part();
+			if (!ip.is_zero())
+				v.push_back(split_ex_to_pair(ip));
+		}
+	return (new add(v, overall_coeff.imag_part()))
+		-> setflag(status_flags::dynallocated);
+}
+
 ex add::eval_ncmul(const exvector & v) const
 {
 	if (seq.empty())
@@ -448,21 +491,23 @@ unsigned add::return_type() const
 	else
 		return seq.begin()->rest.return_type();
 }
-   
-unsigned add::return_type_tinfo() const
+
+return_type_t add::return_type_tinfo() const
 {
 	if (seq.empty())
-		return tinfo_key;
+		return make_return_type_t<add>();
 	else
 		return seq.begin()->rest.return_type_tinfo();
 }
 
-ex add::thisexpairseq(const epvector & v, const ex & oc) const
+// Note: do_index_renaming is ignored because it makes no sense for an add.
+ex add::thisexpairseq(const epvector & v, const ex & oc, bool do_index_renaming) const
 {
 	return (new add(v,oc))->setflag(status_flags::dynallocated);
 }
 
-ex add::thisexpairseq(std::auto_ptr<epvector> vp, const ex & oc) const
+// Note: do_index_renaming is ignored because it makes no sense for an add.
+ex add::thisexpairseq(std::auto_ptr<epvector> vp, const ex & oc, bool do_index_renaming) const
 {
 	return (new add(vp,oc))->setflag(status_flags::dynallocated);
 }
