@@ -3,7 +3,7 @@
  *  Implementation of sequences of expression pairs. */
 
 /*
- *  GiNaC Copyright (C) 1999-2007 Johannes Gutenberg University Mainz, Germany
+ *  GiNaC Copyright (C) 1999-2009 Johannes Gutenberg University Mainz, Germany
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -20,12 +20,6 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#include <iostream>
-#include <algorithm>
-#include <string>
-#include <stdexcept>
-#include <iterator>
-
 #include "expairseq.h"
 #include "lst.h"
 #include "add.h"
@@ -36,11 +30,17 @@
 #include "archive.h"
 #include "operators.h"
 #include "utils.h"
+#include "hash_seed.h"
 #include "indexed.h"
 
+#include <algorithm>
 #if EXPAIRSEQ_USE_HASHTAB
 #include <cmath>
 #endif // EXPAIRSEQ_USE_HASHTAB
+#include <iostream>
+#include <iterator>
+#include <stdexcept>
+#include <string>
 
 namespace GiNaC {
 
@@ -69,9 +69,9 @@ public:
 
 // public
 
-expairseq::expairseq() : inherited(&expairseq::tinfo_static)
+expairseq::expairseq() 
 #if EXPAIRSEQ_USE_HASHTAB
-                                                   , hashtabsize(0)
+	: hashtabsize(0)
 #endif // EXPAIRSEQ_USE_HASHTAB
 {}
 
@@ -108,20 +108,20 @@ void expairseq::copy(const expairseq &other)
 // other constructors
 //////////
 
-expairseq::expairseq(const ex &lh, const ex &rh) : inherited(&expairseq::tinfo_static)
+expairseq::expairseq(const ex &lh, const ex &rh)
 {
 	construct_from_2_ex(lh,rh);
 	GINAC_ASSERT(is_canonical());
 }
 
-expairseq::expairseq(const exvector &v) : inherited(&expairseq::tinfo_static)
+expairseq::expairseq(const exvector &v)
 {
 	construct_from_exvector(v);
 	GINAC_ASSERT(is_canonical());
 }
 
 expairseq::expairseq(const epvector &v, const ex &oc, bool do_index_renaming)
-  : inherited(&expairseq::tinfo_static), overall_coeff(oc)
+  :  overall_coeff(oc)
 {
 	GINAC_ASSERT(is_a<numeric>(oc));
 	construct_from_epvector(v, do_index_renaming);
@@ -129,7 +129,7 @@ expairseq::expairseq(const epvector &v, const ex &oc, bool do_index_renaming)
 }
 
 expairseq::expairseq(std::auto_ptr<epvector> vp, const ex &oc, bool do_index_renaming)
-  : inherited(&expairseq::tinfo_static), overall_coeff(oc)
+  :  overall_coeff(oc)
 {
 	GINAC_ASSERT(vp.get()!=0);
 	GINAC_ASSERT(is_a<numeric>(oc));
@@ -141,11 +141,9 @@ expairseq::expairseq(std::auto_ptr<epvector> vp, const ex &oc, bool do_index_ren
 // archiving
 //////////
 
-expairseq::expairseq(const archive_node &n, lst &sym_lst) : inherited(n, sym_lst)
-#if EXPAIRSEQ_USE_HASHTAB
-	, hashtabsize(0)
-#endif
+void expairseq::read_archive(const archive_node &n, lst &sym_lst) 
 {
+	inherited::read_archive(n, sym_lst);
 	archive_node::archive_node_cit first = n.find_first("rest");
 	archive_node::archive_node_cit last = n.find_last("coeff");
 	++last;
@@ -177,7 +175,6 @@ void expairseq::archive(archive_node &n) const
 	n.add_ex("overall_coeff", overall_coeff);
 }
 
-DEFAULT_UNARCHIVE(expairseq)
 
 //////////
 // functions overriding virtual functions from base classes
@@ -391,12 +388,12 @@ bool expairseq::is_polynomial(const ex & var) const
 	return true;
 }
 
-bool expairseq::match(const ex & pattern, lst & repl_lst) const
+bool expairseq::match(const ex & pattern, exmap & repl_lst) const
 {
 	// This differs from basic::match() because we want "a+b+c+d" to
 	// match "d+*+b" with "*" being "a+c", and we want to honor commutativity
 
-	if (this->tinfo() == ex_to<basic>(pattern).tinfo()) {
+	if (typeid(*this) == typeid(ex_to<basic>(pattern))) {
 
 		// Check whether global wildcard (one that matches the "rest of the
 		// expression", like "*" above) is present
@@ -427,20 +424,10 @@ bool expairseq::match(const ex & pattern, lst & repl_lst) const
 				continue;
 			exvector::iterator it = ops.begin(), itend = ops.end();
 			while (it != itend) {
-				lst::const_iterator last_el = repl_lst.end();
-				--last_el;
 				if (it->match(p, repl_lst)) {
 					ops.erase(it);
 					goto found;
 				}
-				while(true) {
-					lst::const_iterator next_el = last_el;
-					++next_el;
-					if(next_el == repl_lst.end())
-						break;
-					else
-						repl_lst.remove_last();
-				}
 				++it;
 			}
 			return false; // no match found
@@ -458,11 +445,11 @@ found:		;
 			for (size_t i=0; i<num; i++)
 				vp->push_back(split_ex_to_pair(ops[i]));
 			ex rest = thisexpairseq(vp, default_overall_coeff());
-			for (lst::const_iterator it = repl_lst.begin(); it != repl_lst.end(); ++it) {
-				if (it->op(0).is_equal(global_wildcard))
-					return rest.is_equal(it->op(1));
+			for (exmap::const_iterator it = repl_lst.begin(); it != repl_lst.end(); ++it) {
+				if (it->first.is_equal(global_wildcard))
+					return rest.is_equal(it->second);
 			}
-			repl_lst.append(global_wildcard == rest);
+			repl_lst[global_wildcard] = rest;
 			return true;
 
 		} else {
@@ -625,7 +612,7 @@ unsigned expairseq::return_type() const
 
 unsigned expairseq::calchash() const
 {
-	unsigned v = golden_ratio_hash((p_int)this->tinfo());
+	unsigned v = make_hash_seed(typeid(*this));
 	epvector::const_iterator i = seq.begin();
 	const epvector::const_iterator end = seq.end();
 	while (i != end) {
@@ -803,8 +790,8 @@ void expairseq::construct_from_2_ex_via_exvector(const ex &lh, const ex &rh)
 
 void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 {
-	if (ex_to<basic>(lh).tinfo()==this->tinfo()) {
-		if (ex_to<basic>(rh).tinfo()==this->tinfo()) {
+	if (typeid(ex_to<basic>(lh)) == typeid(*this)) {
+		if (typeid(ex_to<basic>(rh)) == typeid(*this)) {
 #if EXPAIRSEQ_USE_HASHTAB
 			unsigned totalsize = ex_to<expairseq>(lh).seq.size() +
 			                     ex_to<expairseq>(rh).seq.size();
@@ -838,7 +825,7 @@ void expairseq::construct_from_2_ex(const ex &lh, const ex &rh)
 #endif // EXPAIRSEQ_USE_HASHTAB
 			return;
 		}
-	} else if (ex_to<basic>(rh).tinfo()==this->tinfo()) {
+	} else if (typeid(ex_to<basic>(rh)) == typeid(*this)) {
 #if EXPAIRSEQ_USE_HASHTAB
 		unsigned totalsize=ex_to<expairseq>(rh).seq.size()+1;
 		if (calc_hashtabsize(totalsize)!=0) {
@@ -1061,7 +1048,7 @@ void expairseq::make_flat(const exvector &v)
 	
 	cit = v.begin();
 	while (cit!=v.end()) {
-		if (ex_to<basic>(*cit).tinfo()==this->tinfo()) {
+		if (typeid(ex_to<basic>(*cit)) == typeid(*this)) {
 			++nexpairseqs;
 			noperands += ex_to<expairseq>(*cit).seq.size();
 		}
@@ -1078,7 +1065,7 @@ void expairseq::make_flat(const exvector &v)
 	make_flat_inserter mf(v, do_idx_rename);
 	cit = v.begin();
 	while (cit!=v.end()) {
-		if (ex_to<basic>(*cit).tinfo()==this->tinfo()) {
+		if (typeid(ex_to<basic>(*cit)) == typeid(*this)) {
 			ex newfactor = mf.handle_factor(*cit, _ex1);
 			const expairseq &subseqref = ex_to<expairseq>(newfactor);
 			combine_overall_coeff(subseqref.overall_coeff);
@@ -1113,7 +1100,7 @@ void expairseq::make_flat(const epvector &v, bool do_index_renaming)
 	
 	cit = v.begin();
 	while (cit!=v.end()) {
-		if (ex_to<basic>(cit->rest).tinfo()==this->tinfo()) {
+		if (typeid(ex_to<basic>(cit->rest)) == typeid(*this)) {
 			++nexpairseqs;
 			noperands += ex_to<expairseq>(cit->rest).seq.size();
 		}
@@ -1131,7 +1118,7 @@ void expairseq::make_flat(const epvector &v, bool do_index_renaming)
 	// copy elements and split off numerical part
 	cit = v.begin();
 	while (cit!=v.end()) {
-		if (ex_to<basic>(cit->rest).tinfo()==this->tinfo() &&
+		if ((typeid(ex_to<basic>(cit->rest)) == typeid(*this)) &&
 		    this->can_make_flat(*cit)) {
 			ex newrest = mf.handle_factor(cit->rest, cit->coeff);
 			const expairseq &subseqref = ex_to<expairseq>(newrest);
